#ifndef _gen_mozilla_idl_generated_events_h_
#define _gen_mozilla_idl_generated_events_h_

/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */
#include "nscore.h"
class nsIDOMEvent;
class nsPresContext;
namespace mozilla {
class WidgetEvent;
namespace dom {
class EventTarget;
}
}

nsresult
NS_NewDOMMozSettingsEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMCustomEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMPageTransitionEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMDOMTransactionEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMPopStateEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMHashChangeEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMCloseEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMDeviceOrientationEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMMozApplicationEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMSmartCardEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMStyleRuleChangeEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMStyleSheetChangeEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMStyleSheetApplicableStateChangeEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMElementReplaceEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMMozSmsEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMMozMmsEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMSpeechSynthesisEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMDeviceStorageChangeEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMPopupBlockedEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMRecordErrorEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);
nsresult
NS_NewDOMSpeechRecognitionEvent(nsIDOMEvent** aInstance, mozilla::dom::EventTarget* aOwner, nsPresContext* aPresContext, mozilla::WidgetEvent* aEvent);

#endif
