/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIDOMCSSStyleSheet;

[builtinclass, uuid(75f27960-fd1a-4e05-8b8b-55d97ea22a9a)]
interface nsIDOMStyleSheetApplicableStateChangeEvent : nsIDOMEvent
{
  readonly attribute nsIDOMCSSStyleSheet stylesheet;
  readonly attribute boolean applicable;
  [noscript] void initStyleSheetApplicableStateChangeEvent(in DOMString aTypeArg,
                                                           in boolean aCanBubbleArg,
                                                           in boolean aCancelableArg,
                                                           in nsIDOMCSSStyleSheet aStyleSheet,
                                                           in boolean aApplicable);
};
