/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIAutofillController.idl
 */

#ifndef __gen_nsIAutofillController_h__
#define __gen_nsIAutofillController_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIDOMHTMLFormElement; /* forward declaration */

class nsIDOMWindow; /* forward declaration */


/* starting interface:    nsIAutofillController */
#define NS_IAUTOFILLCONTROLLER_IID_STR "cbf47f9d-a13d-4fad-8221-8964086b1b8a"

#define NS_IAUTOFILLCONTROLLER_IID \
  {0xcbf47f9d, 0xa13d, 0x4fad, \
    { 0x82, 0x21, 0x89, 0x64, 0x08, 0x6b, 0x1b, 0x8a }}

class NS_NO_VTABLE nsIAutofillController : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IAUTOFILLCONTROLLER_IID)

  /* void requestAutocomplete (in nsIDOMHTMLFormElement form, in nsIDOMWindow window); */
  NS_IMETHOD RequestAutocomplete(nsIDOMHTMLFormElement *form, nsIDOMWindow *window) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIAutofillController, NS_IAUTOFILLCONTROLLER_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIAUTOFILLCONTROLLER \
  NS_IMETHOD RequestAutocomplete(nsIDOMHTMLFormElement *form, nsIDOMWindow *window); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIAUTOFILLCONTROLLER(_to) \
  NS_IMETHOD RequestAutocomplete(nsIDOMHTMLFormElement *form, nsIDOMWindow *window) { return _to RequestAutocomplete(form, window); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIAUTOFILLCONTROLLER(_to) \
  NS_IMETHOD RequestAutocomplete(nsIDOMHTMLFormElement *form, nsIDOMWindow *window) { return !_to ? NS_ERROR_NULL_POINTER : _to->RequestAutocomplete(form, window); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsAutofillController : public nsIAutofillController
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIAUTOFILLCONTROLLER

  nsAutofillController();

private:
  ~nsAutofillController();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsAutofillController, nsIAutofillController)

nsAutofillController::nsAutofillController()
{
  /* member initializers and constructor code */
}

nsAutofillController::~nsAutofillController()
{
  /* destructor code */
}

/* void requestAutocomplete (in nsIDOMHTMLFormElement form, in nsIDOMWindow window); */
NS_IMETHODIMP nsAutofillController::RequestAutocomplete(nsIDOMHTMLFormElement *form, nsIDOMWindow *window)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIAutofillController_h__ */
